#!/usr/bin/env python

##*****************************************************************************
##
## File: readAudio.py
##
## Copyright (c) 2016 Autodesk, Inc.
## All rights reserved.
##
## Use of this software is subject to the terms of the Autodesk license
## agreement provided at the time of installation or download, or which
## otherwise accompanies this software in either electronic or hard copy form.
##
##*****************************************************************************

"""
Wiretap SDK sample python program to read all samples of an audio clip.
"""

from __future__ import print_function
from __future__ import absolute_import

import sys
import getopt

from adsk.libwiretapPythonClientAPI import WireTapClient
from adsk.libwiretapPythonClientAPI import WireTapServerHandle
from adsk.libwiretapPythonClientAPI import WireTapNodeHandle
from adsk.libwiretapPythonClientAPI import WireTapClipFormat
from adsk.libwiretapPythonClientAPI import WireTapAudioFormat
from adsk.libwiretapPythonClientAPI import WireTapInt

class WireTapException(Exception):
    pass

def main(argv):

    # check the command line options for correctness
    try:
        opts, _ = getopt.getopt(argv, "h:n:")
    except getopt.GetoptError:
        print("Usage:\n" \
              "-h <hostname>\n" \
              "-n <node Id>")
        sys.exit(2)

    # default host name and node ID
    host_name = "localhost"
    node_id = ""

    # parse command line option to set specified host name
    # and node ID
    for opt, arg in opts:
        if opt == '-h':
            host_name = arg
        elif opt == '-n':
            node_id = arg

    # Initialize the Wiretap Client API.
    #
    wiretap_client = WireTapClient()
    if not wiretap_client.init():
        raise WireTapException("Unable to initialize Wiretap client API.")

    read_audio(host_name, node_id)

def read_audio(host_name, node_id):

    # Instantiate a server handle
    #
    server_handle = WireTapServerHandle(host_name)

    # Instantiate the clip node handle for the given server and node ID.
    #
    clip_node_handle = WireTapNodeHandle(server_handle, node_id)

    # Get the list of audio blocks.  Each frame is a block of samples.  The
    # server decides the size of the audio blocks.
    #
    num_frames = WireTapInt()
    if not clip_node_handle.getNumFrames(num_frames):
        raise WireTapException("Unable to obtain number of frames: %s." % \
                               clip_node_handle.lastError())

    # Get the clip format.  This is required to know how to interpret the frame
    # data, and to know how big the frame buffer must be.
    #
    clip_format = WireTapAudioFormat()
    if not clip_node_handle.getClipFormat(clip_format):
        raise WireTapException("Unable to obtain clip format: %s." % \
                               clip_node_handle.lastError())

    # Interpret the format tag (see WireTapTypes.h for a list of tags).
    #
    if clip_format.formatTag() == WireTapClipFormat.FORMAT_DL_AUDIO_FLOAT():
        print("Audio clip contains floating point samples.")
    elif clip_format.formatTag() == WireTapClipFormat.FORMAT_DL_AUDIO_INT24():
        print("Audio clip contains 24 bit integer samples.")
    else:
        print("Unsupported clip format: %s." % clip_format.formatTag())

    # Audio samples are interlaced across channels.
    #
    print("Audio stream has %d channels." % clip_format.numChannels())

    # The number of samples is stored in the clip format's width.
    #
    print("Audio stream has %d samples per channel." % clip_format.numSamples())

    # Allocate the audio block sample buffer.  See sections above to determine
    # how to interpret the raw buffers.
    #
    buff = '0' * clip_format.frameBufferSize()

    # Read all blocks of samples.
    #
    for frame_index in range(0, num_frames):
        print("Reading sample block %i." % frame_index)
        if not clip_node_handle.readFrame(
                frame_index,
                buff,
                clip_format.frameBufferSize()):
            raise WireTapException(
                "Unable to obtain read sample block %i: %s." % \
                (frame_index, clip_node_handle.lastError()))

if __name__ == '__main__':
    main(sys.argv[1:])
