#!/usr/bin/env python

##*****************************************************************************
##
## File: readFrames.py
##
## Copyright (c) 2016 Autodesk, Inc.
## All rights reserved.
##
## Use of this software is subject to the terms of the Autodesk license
## agreement provided at the time of installation or download, or which
## otherwise accompanies this software in either electronic or hard copy form.
##
##*****************************************************************************

"""
Wiretap SDK sample python program to read all frames of a clip.
"""

from __future__ import print_function
from __future__ import absolute_import

import sys
import getopt

from adsk.libwiretapPythonClientAPI import WireTapClient
from adsk.libwiretapPythonClientAPI import WireTapServerHandle
from adsk.libwiretapPythonClientAPI import WireTapNodeHandle
from adsk.libwiretapPythonClientAPI import WireTapClipFormat
from adsk.libwiretapPythonClientAPI import WireTapInt

class WireTapException(Exception):
    pass

def main(argv):

    # check the command line options for correctness
    try:
        opts, _ = getopt.getopt(argv, "h:n:")
    except getopt.GetoptError:
        print("Usage:\n" \
              "-h <hostname>\n" \
              "-n <node Id>")
        sys.exit(2)

    # default host name and node ID
    host_name = "localhost"
    node_id = ""

    # parse command line option to set specified host name
    # and node ID
    for opt, arg in opts:
        if opt == '-h':
            host_name = arg
        elif opt == '-n':
            node_id = arg

    # Initialize the Wiretap Client API.
    #
    wiretap_client = WireTapClient()
    if not wiretap_client.init():
        raise WireTapException("Unable to initialize Wiretap client API.")

    read_frames(host_name, node_id)

def read_frames(host_name, node_id):

    # Instantiate a server handle
    #
    server_handle = WireTapServerHandle(host_name)

    # Instantiate the clip node handle for the given server and node ID.
    #
    clip = WireTapNodeHandle(server_handle, node_id)

    # Get the number of frames in the clip.
    #
    num_frames = WireTapInt()
    if not clip.getNumFrames(num_frames):
        raise WireTapException("Unable to obtain number of frames: %s." % \
                               clip.lastError())

    # Get the clip format.  This is required to know how to interpret the frame
    # data, and to know how big the frame buffer must be.
    #
    fmt = WireTapClipFormat()
    if not clip.getClipFormat(fmt):
        raise WireTapException("Unable to obtain clip format: %s." % \
                               clip.lastError())

    # Interpret the format tag (see WireTapTypes.h for a list of tags).
    #
    if fmt.formatTag() == WireTapClipFormat.FORMAT_RGB():
        print("This is an RGB clip.")
    elif fmt.formatTag() == WireTapClipFormat.FORMAT_YUV():
        print("This is a YUV clip.")
    else:
        print("Unknown clip format: %s." % fmt.formatTag())

    # buff is a buffer initialized to zero
    #
    buff = '0' * fmt.frameBufferSize()

    # Read all frames.
    #
    for frame_number in range(0, num_frames):
        print("Reading frame %i." % frame_number)
        if not clip.readFrame(frame_number, buff, fmt.frameBufferSize()):
            raise WireTapException("Unable to obtain read frame %i: %s." % \
                                   (frame_number, clip.lastError()))

        # Print the frame info.
        #
        print("Successfully read frame %i." % frame_number)

if __name__ == '__main__':
    main(sys.argv[1:])
