#!/usr/bin/env python

##*****************************************************************************
##
## File: resolvesStorageId.py
##
## Copyright (c) 2016 Autodesk, Inc.
## All rights reserved.
##
## Use of this software is subject to the terms of the Autodesk license
## agreement provided at the time of installation or download, or which
## otherwise accompanies this software in either electronic or hard copy form.
##
##*****************************************************************************

"""
Wiretap SDK python sample program to resolve a storage ID into a Wiretap server.
"""

from __future__ import print_function
from __future__ import absolute_import

import sys
import getopt

from adsk.libwiretapPythonClientAPI import WireTapClient
from adsk.libwiretapPythonClientAPI import WireTapServerId
from adsk.libwiretapPythonClientAPI import WireTapServerList
from adsk.libwiretapPythonClientAPI import WireTapServerInfo

class WireTapException(Exception):
    pass

def main(argv):

    # check the command line options for correctness
    try:
        opts, _ = getopt.getopt(argv, "s:")
    except getopt.GetoptError:
        print("Usage:\n" \
              "-s <storage Id>\n")
        sys.exit(2)

    storage_id = ""

    # parse command line option to set specified storage id.
    for opt, arg in opts:
        if opt == '-s':
            storage_id = arg

    # Initialize the Wiretap Client API.
    #
    wiretap_client = WireTapClient()
    if not wiretap_client.init():
        raise WireTapException("Unable to initialize Wiretap client API.")

    resolve_storage_id(storage_id)

def resolve_storage_id(storage_id):
    """
    Resolve the given storage ID to determine to which Wiretap server the
    storage device is connected
    """

    server_list = WireTapServerList()
    server_id = WireTapServerId()
    server_info = WireTapServerInfo()

    server_id.setStorageId(storage_id)
    if not server_list.resolve(server_id, server_info):
        raise WireTapException("Unable to resolve '%s' into a server ID: %s." % \
                               (storage_id, server_list.lastError()))

    # Obtain IP address of the Wiretap server
    #
    print("Successfuly resolved storage ID '%s' into server '%s'." % \
          (storage_id, server_info.getIPAddr()))

    print("Server display name : '%s'." % server_info.getDisplayName())

if __name__ == '__main__':
    main(sys.argv[1:])
