#!/usr/bin/env python

#
# Wiretap SDK tool for reading frames from a media file into a buffer, and
# writing the frames from the buffer into a clip.
#

from __future__ import print_function

import getopt
import string
import sys
from libwiretapPythonClientAPI import (
    WireTapClient,
    WireTapServerHandle,
    WireTapNodeHandle,
    WireTapClipFormat,
    WireTapInt
)

writeFlag = 0
skipFileIOFlag = 0

# Define a local Clip class with several methods:
#   - init       (instantiates a clip node, gets clip format, and opens file)
#   - numFrames  (gets number of frames defined for clip node)
#   - readFrame  (reads a single frame from media file to buffer,
#                 and writes from buffer to the clip node)
#
class Clip(object):

    # init method (constructor)
    #
    def __init__(self, server, nodeId, frameFile):
        self.node_ = WireTapNodeHandle(server, nodeId)

        fmt = WireTapClipFormat()
        if not self.node_.getClipFormat(fmt):
            raise Exception("Unable to get clip format for node '%s': %s " % \
                            (nodeId, self.node_.lastError()))

        self.bufSize_ = fmt.frameBufferSize()
        self.buf_ = '0'*fmt.frameBufferSize()
        self.numFrames_ = WireTapInt()

        if not self.node_.getNumFrames(self.numFrames_):
            raise Exception("Unable to get number of frames for node '%s': %s " % \
                            (nodeId, self.node_.lastError()))

        if skipFileIOFlag == 0:
            if writeFlag:
                self.fileIO_ = open(frameFile, 'rb')
            else:
                self.fileIO_ = open(frameFile, 'wb')

    # numFrames method
    #
    def numFrames(self):
        return self.numFrames_

    # readFrame method
    #
    def readFrame(self, frameIndex):

        if frameIndex >= self.numFrames():
            return

        if writeFlag:

            #Read in the frame file.
            #
            if skipFileIOFlag == 0:
                self.buf_ = self.fileIO_.read()

            # Write the buffer.
            #
            if not self.node_.writeFrame(frameIndex, self.buf_, self.bufSize_):
                raise Exception("Unable to write frame %d: %s " % \
                                 (frameIndex, self.node_.lastError()))
            output = "written"

        else:
            if not self.node_.readFrame(frameIndex, self.buf_, self.bufSize_):
                raise Exception("Unable to read frame %d: %s " % \
                                (frameIndex, self.node_.lastError()))

            # Write the frame file.
            #
            if skipFileIOFlag == 0:
                self.fileIO_.write(self.buf_)
            output = "read"

        # Spit out result.
        #
        print("Frame %d of clip %s successfully %s. " % (
            frameIndex, self.node_.getNodeId().id(), output
        ))

# printUsage method
#
def printUsage():
    print("Usage:\n"
          "  -n <node Id (can be multiple IDs separated by commas)>\n"
          "  -f <frame file (default = ./frame_#.format)>\n"
          "  -s skip file IO (default = false)\n"
          "  -i <zero-based frame index, -1 for all frames> (default = 0)\n"
          "  -w write frame (default = read)\n"
          "  -h <host name or IP address> (default = localhost)\n")

# main method
#
def main():

    # Check the command line options for correctness.
    try:
        opts, _ = getopt.getopt(sys.argv, "h:f:n:s:i:w")
    except getopt.GetoptError:
        printUsage()
        sys.exit(2)

    # Declare variables (to be set from command line arguments).
    # For hostName, the default "localhost" will work if a Wiretap server
    # is running on your machine.
    #
    hostName = "localhost"
    nodes = ""
    fileName = "frame.raw"
    frameIndex = 0

    # Declare and initialize global variables for keeping track of whether
    # frame has been written
    global writeFlag
    global skipFileIOFlag

    # Parse command line and set variables declared earlier.
    #
    for opt, arg in opts:
        if opt == '-h':
            hostName = arg
        elif opt == '-n':
            nodes = arg
        elif opt == '-i':
            frameIndex = int(arg)
        elif opt == '-f':
            fileName = arg
        elif opt == '-s':
            skipFileIOFlag = 1
        elif opt == '-w':
            writeFlag = 1

    if nodes == "":
        print("Node ID not specified.\n")
        printUsage()
        sys.exit(2)


    # Initialize the Wiretap Client API.
    #
    wiretapClient = WireTapClient()
    if not wiretapClient.init():
        raise Exception("Unable to initialize Wiretap client API")

    try:
        # Instantiate a server handle
        #
        server = WireTapServerHandle(hostName)

        # Parse the string of node IDs (entered on the command line),
        # create an object of the local Clip class for each node ID,
        # and store the Clip objects in a list.
        #
        clips = []
        i = 0

        nodeIdStrings = nodes.split(',')
        for nodeId in nodeIdStrings:
            fileNameSplit = fileName.split(".")
            fileNameSplit.insert(-1, "_" + str(i) + ".")
            fileName = string.join(fileNameSplit, '')

            clips.append(Clip(server, nodeId, fileName))
            i += 1

        if clips:
            raise Exception("Must specify at least one clip node ID.")

        # Get the maximum number of frames across all clips.
        #
        numFrames = clips[0].numFrames()
        for clip in clips:
            numFrames = max(clip.numFrames(), numFrames)

        # For each clip, read frames from media file and write to the clip.
        #
        # If a frameIndex of -1 was entered on the command line,
        # write all frames to a clip.
        #
        if frameIndex < 0:
            for frame in range(numFrames):
                for clip in clips:
                    clip.readFrame(frame)
        #
        # If a particular frameIndex was entered on the command line,
        # read frame and write it at specified index in the clip.
        #
        else:
            for clip in clips:
                clip.readFrame(frameIndex)

    finally:
        # Must destroy WireTapServerHandle and WireTapNodeHandle before
        # uninitializing the Wiretap Client API.
        #
        clips = None
        server = None


if __name__ == '__main__':
    main()
